//////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/SceneGenerator/BasicImpl/CallbackInfo.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * RTTC::SceneGenerator::BasicImpl::CallbackInfo.
 *
 * This file contains the declaration of and documentation for the
 * RTTC::SceneGenerator::BasicImpl::CallbackInfo class.
 */

#ifndef RTTC_SceneGenerator_BasicImpl_CallbackInfo_h
#define RTTC_SceneGenerator_BasicImpl_CallbackInfo_h

#include <DCT/System/config.h>
#include <basicImpl_RTTC_TASemulator_v4_EXPORT.h>
#include <DCT/Utils/SmartPtr.h>
#include <RTTC/SceneGenerator/Proxy.h>
#include <list>
#include <ostream>

namespace RTTC
{
  namespace SceneGenerator
  {
    namespace BasicImpl
    {
      /*! \class CallbackInfo CallbackInfo.h \
         RTTC/SceneGenerator/BasicImpl/CallbackInfo.h
       *
       * \brief CallbackInfo object passed to all callbacks
       *
       * This is a convenience class to simplify access to the callbacks.
       * Whatever state is encapsulated here is accessible to all the
       * callbacks.  A smart pointer is passed to all the factories,
       * which in turn pass it to created callbacks.
       *
       * The basic implementation owns a std::list of ProxyPtrs.  The
       * discovery callback sticks the newly discovered ProxyPtr onto
       * the list, and the destruction callback pulls it off.  This
       * implementation is also initialized with a reference to an
       * OutputStream, which can be used to represent a view object
       * that can be updated during state change.
       *
       * A user application may modify the CallbackInfo in any way desired,
       * and the encapsulated information will be available to all callbacks.
       */
      class basicImpl_RTTC_TASemulator_v4_EXPORT CallbackInfo
      {
      public:
        /*!
          \param[in] os A std::ostream to which output will be written.
          \param[in] verbosity Controls the amount of output generated.
        */
        CallbackInfo(
          std::ostream & os,
          unsigned int verbosity = 0 );

        //! Instances live for at least the duration of a subscription.
        ~CallbackInfo();

        //! Return a reference to the outputstream.
        std::ostream &
        getOutputStream();

        //! Return a reference to the list maintained in the CallbackInfo.
        std::list< RTTC::SceneGenerator::ProxyPtr > &
        getDiscoveredSDOlist();

        //! Return the verbosity level to be used for output
        /*!
         * 0 - No output
         * 1 - Callback evoked
         * 2 - 1 plus publication state
         */
        unsigned int
        getVerbosity();

      private:
        //! Default copy construction is disallowed.
        CallbackInfo( CallbackInfo const & rhs );
        //! Default copy assignment is disallowed.
        CallbackInfo & operator=( CallbackInfo const & rhs );

        //! Hold a reference to an ostream
        std::ostream & _outputStream;

        //! Hold a list of the discovered SDO ProxyPtrs
        std::list< RTTC::SceneGenerator::ProxyPtr >
          _discoveredSDOlist;

        //! Specify the output verbosity
        unsigned int _verbosity;
      };

      /*! \brief A reference-counted "smart" pointer to a
       * CallbackInfo.
       *
       * TENA uses a variety of smart pointers to simplify the management
       * of memory associated with objects returned by the middleware. This
       * type is constructed from a raw call to new() so that user code
       * cannot easily access a deleted object or leak an object.
       *
       * Because smart pointers are incomplete types without the full
       * definition of the pointed-to object, these smart pointers
       * are always defined in the same header as the pointed-to object
       */
      typedef DCT::Utils::SmartPtr< CallbackInfo > CallbackInfoPtr;

    } // End of namespace BasicImpl
  } // End of namespace SceneGenerator
} // End of namespace RTTC

/****************************************************************************/
/*** Inline definition of REALLY simple methods                     *********/
/****************************************************************************/

///////////////////////////////////////////////////////////////////////////////
inline
std::list< RTTC::SceneGenerator::ProxyPtr > &
RTTC::SceneGenerator::BasicImpl::
CallbackInfo::
getDiscoveredSDOlist()
{
  return _discoveredSDOlist;
}

///////////////////////////////////////////////////////////////////////////////
inline
std::ostream &
RTTC::SceneGenerator::BasicImpl::
CallbackInfo::
getOutputStream()
{
  return _outputStream;
}

///////////////////////////////////////////////////////////////////////////////
inline
unsigned int
RTTC::SceneGenerator::BasicImpl::
CallbackInfo::
getVerbosity()
{
  return _verbosity;
}

#endif // ifndef RTTC_SceneGenerator_BasicImpl_CallbackInfo_h
